@@ -4,8 +4,20 @@ from pathlib import Path
 from django.db.models import Q
 from django.db.models.functions import Now
 
-from codex.librarian.queue_mp import LIBRARIAN_QUEUE, BulkComicCoverCreateTask
-from codex.models import Comic, Credit, Folder, Imprint, Publisher, Series, Volume
+from codex.librarian.covers.tasks import CoverRemoveTask
+from codex.librarian.db.status import ImportStatusTypes
+from codex.librarian.queue_mp import LIBRARIAN_QUEUE
+from codex.librarian.status_control import StatusControl
+from codex.models import (
+    Comic,
+    Credit,
+    Folder,
+    Imprint,
+    Publisher,
+    Series,
+    Timestamp,
+    Volume,
+)
 from codex.settings.logging import get_logger
 
 
@@ -51,11 +63,7 @@ def _link_comic_fks(md, library, path):
         name=volume_name,
     )
     parent_path = Path(path).parent
-    if parent_path == Path(library.path):
-        parent_folder = None
-    else:
-        parent_folder = Folder.objects.get(path=parent_path)
-    md["parent_folder"] = parent_folder
+    md["parent_folder"] = Folder.objects.get(path=parent_path)
 
 
 def _update_comics(library, comic_paths, mds):
@@ -67,7 +75,7 @@ def _update_comics(library, comic_paths, mds):
     LOG.verbose(f"Preparing {num_comics} comics for update in library {library.path}.")
     # Get existing comics to update
     comics = Comic.objects.filter(library=library, path__in=comic_paths).only(
-        "pk", "path", *BULK_UPDATE_COMIC_FIELDS
+        *BULK_UPDATE_COMIC_FIELDS
     )
 
     # set attributes for each comic
@@ -91,7 +99,8 @@ def _update_comics(library, comic_paths, mds):
 
     LOG.verbose(f"Bulk updating {num_comics} comics.")
     count = Comic.objects.bulk_update(update_comics, BULK_UPDATE_COMIC_FIELDS)
-    task = BulkComicCoverCreateTask(True, tuple(comic_pks))
+    task = CoverRemoveTask(frozenset(comic_pks))
+    LOG.verbose(f"Purging covers for {len(comic_pks)} updated comics.")
     LIBRARIAN_QUEUE.put(task)
     return count
 
@@ -124,13 +133,7 @@ def _create_comics(library, comic_paths, mds):
     num_comics = len(create_comics)
     LOG.verbose(f"Bulk creating {num_comics} comics...")
     created_comics = Comic.objects.bulk_create(create_comics)
-    # Start the covers task.
-    comic_pks = []
-    for comic in created_comics:
-        comic_pks.append(comic.pk)
-    task = BulkComicCoverCreateTask(True, tuple(comic_pks))
-    LIBRARIAN_QUEUE.put(task)
-    return num_comics
+    return len(created_comics)
 
 
 def _link_folders(folder_paths):
@@ -140,7 +143,7 @@ def _link_folders(folder_paths):
     folder_pks = Folder.objects.filter(path__in=folder_paths).values_list(
         "pk", flat=True
     )
-    return set(folder_pks)
+    return frozenset(folder_pks)
 
 
 def _link_credits(credits):
@@ -155,7 +158,7 @@ def _link_credits(credits):
         }
         filter = filter | Q(**filter_dict)
     credit_pks = Credit.objects.filter(filter).values_list("pk", flat=True)
-    return set(credit_pks)
+    return frozenset(credit_pks)
 
 
 def _link_named_m2ms(all_m2m_links, comic_pk, md):
@@ -168,13 +171,13 @@ def _link_named_m2ms(all_m2m_links, comic_pk, md):
         pks = related_model.objects.filter(name__in=names).values_list("pk", flat=True)
         if field not in all_m2m_links:
             all_m2m_links[field] = {}
-        all_m2m_links[field][comic_pk] = set(pks)
+        all_m2m_links[field][comic_pk] = frozenset(pks)
 
 
 def _link_comic_m2m_fields(m2m_mds):
     """Get the complete m2m field data to create."""
     all_m2m_links = {}
-    comic_paths = set(m2m_mds.keys())
+    comic_paths = frozenset(m2m_mds.keys())
     LOG.verbose(
         f"Preparing {len(comic_paths)} comics for many to many relation recreation."
     )
@@ -206,7 +209,7 @@ def bulk_recreate_m2m_field(field_name, m2m_links):
     Since we can't bulk_update or bulk_create m2m fields use a trick.
     bulk_create() on the through table:
     https://stackoverflow.com/questions/6996176/how-to-create-an-object-for-a-django-model-with-a-many-to-many-field/10116452#10116452 # noqa: B950,E501
-    https://docs.djangoproject.com/en/3.2/ref/models/fields/#django.db.models.ManyToManyField.through # noqa: B950,E501
+    https://docs.djangoproject.com/en/4.0/ref/models/fields/#django.db.models.ManyToManyField.through # noqa: B950,E501
     """
     LOG.verbose(f"Recreating {field_name} relations for altered comics.")
     field = getattr(Comic, field_name)
@@ -233,21 +236,55 @@ def bulk_recreate_m2m_field(field_name, m2m_links):
 def bulk_import_comics(library, create_paths, update_paths, all_bulk_mds, all_m2m_mds):
     """Bulk import comics."""
     if not (create_paths or update_paths or all_bulk_mds or all_m2m_mds):
+        StatusControl.finish_many(
+            (
+                ImportStatusTypes.FILES_MODIFIED,
+                ImportStatusTypes.FILES_CREATED,
+                ImportStatusTypes.LINK_M2M_FIELDS,
+            )
+        )
         return 0
 
-    update_count = _update_comics(library, update_paths, all_bulk_mds)
-    create_count = _create_comics(library, create_paths, all_bulk_mds)
-
-    all_m2m_links = _link_comic_m2m_fields(all_m2m_mds)
-    for field_name, m2m_links in all_m2m_links.items():
+    try:
         try:
-            bulk_recreate_m2m_field(field_name, m2m_links)
-        except Exception as exc:
-            LOG.error(f"Error recreating m2m field: {field_name}")
-            LOG.exception(exc)
+            try:
+                StatusControl.start(
+                    ImportStatusTypes.FILES_MODIFIED, name=f"({len(update_paths)})"
+                )
+                update_count = _update_comics(library, update_paths, all_bulk_mds)
+            finally:
+                StatusControl.finish(ImportStatusTypes.FILES_MODIFIED)
+            StatusControl.start(
+                ImportStatusTypes.FILES_CREATED, name=f"({len(create_paths)})"
+            )
+            create_count = _create_comics(library, create_paths, all_bulk_mds)
+        finally:
+            StatusControl.finish(ImportStatusTypes.FILES_CREATED)
+            # Just to be sure.
+        all_m2m_links = _link_comic_m2m_fields(all_m2m_mds)
+        total_links = 0
+        for m2m_links in all_m2m_links.values():
+            total_links += len(m2m_links)
+        StatusControl.start(ImportStatusTypes.LINK_M2M_FIELDS, total_links)
+
+        for m2m_links in all_m2m_links.values():
+            completed_links = 0
+            for field_name, m2m_links in all_m2m_links.items():
+                try:
+                    bulk_recreate_m2m_field(field_name, m2m_links)
+                except Exception as exc:
+                    LOG.error(f"Error recreating m2m field: {field_name}")
+                    LOG.exception(exc)
+                completed_links = len(m2m_links)
+                StatusControl.update(
+                    ImportStatusTypes.LINK_M2M_FIELDS, completed_links, total_links
+                )
+    finally:
+        StatusControl.finish(ImportStatusTypes.LINK_M2M_FIELDS)
 
     update_log = f"Updated {update_count} Comics."
     if update_count:
+        Timestamp.touch(Timestamp.COVERS)
         LOG.info(update_log)
     else:
         LOG.verbose(update_log)
