@@ -4,7 +4,6 @@ import time
 from pathlib import Path
 
 from django.core.cache import cache
-from django.db.models.functions import Now
 from humanize import precisedelta
 
 from codex.librarian.db.aggregate_metadata import get_aggregate_metadata
@@ -14,15 +13,13 @@ from codex.librarian.db.deleted import bulk_comics_deleted, bulk_folders_deleted
 from codex.librarian.db.failed_imports import bulk_fail_imports
 from codex.librarian.db.moved import bulk_comics_moved, bulk_folders_moved
 from codex.librarian.db.query_fks import query_all_missing_fks
-from codex.librarian.queue_mp import (
-    LIBRARIAN_QUEUE,
-    AdminNotifierTask,
-    BroadcastNotifierTask,
-    DBDiffTask,
-    DelayedTasks,
-    SearchIndexUpdateTask,
-)
+from codex.librarian.db.status import ImportStatusTypes
+from codex.librarian.db.tasks import UpdaterDBDiffTask
+from codex.librarian.queue_mp import LIBRARIAN_QUEUE, DelayedTasks
+from codex.librarian.search.tasks import SearchIndexJanitorUpdateTask
+from codex.librarian.status_control import StatusControl
 from codex.models import Library
+from codex.notifier.tasks import FAILED_IMPORTS_TASK, LIBRARY_CHANGED_TASK
 from codex.settings.logging import LOG_EVERY, VERBOSE, get_logger
 from codex.threads import QueuedThread
 
@@ -31,7 +28,7 @@ WRITE_WAIT_EXPIRY = LOG_EVERY
 LOG = get_logger(__name__)
 
 
-def _wait_for_filesystem_ops_to_finish(task: DBDiffTask) -> bool:
+def _wait_for_filesystem_ops_to_finish(task: UpdaterDBDiffTask) -> bool:
     """Watchdog sends events before filesystem events finish, so wait for them."""
     started_checking = time.time()
 
@@ -105,7 +102,7 @@ def _batch_modified_and_created(
     )
     new_failed_imports = bulk_fail_imports(library, fis)
     changed |= imported_count > 0
-    return changed, imported_count, new_failed_imports
+    return changed, imported_count, bool(new_failed_imports)
 
 
 def _log_task(path, task):
@@ -139,55 +136,94 @@ def _log_task(path, task):
         LOG.verbose("  " + log)
 
 
+def _init_librarian_status(task, path):
+    """Update the librarian status tasks."""
+    types_map = {}
+    if task.files_moved:
+        types_map[ImportStatusTypes.FILES_MOVED] = ({"total": len(task.files_moved)},)
+    if task.files_modified or task.files_created:
+        total_paths = len(task.files_modified) + len(task.files_created)
+        types_map[ImportStatusTypes.AGGREGATE_TAGS] = {
+            "total": total_paths,
+            "name": path,
+        }
+        types_map[ImportStatusTypes.QUERY_MISSING_FKS] = {}
+        types_map[ImportStatusTypes.CREATE_FKS] = {}
+        if task.files_modified:
+            types_map[ImportStatusTypes.FILES_MODIFIED] = {
+                "name": f"({len(task.files_modified)})"
+            }
+        if task.files_created:
+            types_map[ImportStatusTypes.FILES_CREATED] = {
+                "name": f"({len(task.files_created)})"
+            }
+        if task.files_modified or task.files_created:
+            types_map[ImportStatusTypes.LINK_M2M_FIELDS] = {}
+    if task.files_deleted:
+        types_map[ImportStatusTypes.FILES_DELETED] = {
+            "name": f"({len(task.files_deleted)})"
+        }
+    StatusControl.start_many(types_map)
+
+
+def _finish_apply(library):
+    """Finish all librarian statuses."""
+    library.update_in_progress = False
+    library.save()
+    StatusControl.finish_many(ImportStatusTypes.values())
+
+
 def _apply(task):
     """Bulk import comics."""
     start_time = time.time()
     library = Library.objects.get(pk=task.library_id)
-    _log_task(library.path, task)
-    library.update_in_progress = True
-    library.save()
-    LIBRARIAN_QUEUE.put(AdminNotifierTask("LIBRARY_UPDATE_IN_PROGRESS"))
+    try:
+        library.update_in_progress = True
+        library.save()
+        too_long = _wait_for_filesystem_ops_to_finish(task)
+        if too_long:
+            LOG.warning(
+                "Import apply waited for the filesystem to stop changing too long. "
+                "Try polling again once files have finished copying."
+            )
+            return
+        _log_task(library.path, task)
+        _init_librarian_status(task, library.path)
 
-    too_long = _wait_for_filesystem_ops_to_finish(task)
-    if too_long:
-        LOG.warning(
-            "Import apply waited for the filesystem to stop changing too long. "
-            "Try polling again once files have finished copying."
+        changed = False
+        changed |= bulk_folders_moved(library, task.dirs_moved)
+        changed |= bulk_comics_moved(library, task.files_moved)
+        changed |= bulk_folders_modified(library, task.dirs_modified)
+        (
+            changed_comics,
+            imported_count,
+            new_failed_imports,
+        ) = _batch_modified_and_created(
+            library, task.files_modified, task.files_created
         )
-
-    changed = False
-    changed |= bulk_folders_moved(library, task.dirs_moved)
-    changed |= bulk_comics_moved(library, task.files_moved)
-    changed |= bulk_folders_modified(library, task.dirs_modified)
-    changed_comics, imported_count, new_failed_imports = _batch_modified_and_created(
-        library, task.files_modified, task.files_created
-    )
-    changed |= changed_comics
-    changed |= bulk_folders_deleted(library, task.dirs_deleted)
-    changed |= bulk_comics_deleted(library, task.files_deleted)
-    cache.clear()
-
-    Library.objects.filter(pk=task.library_id).update(
-        update_in_progress=False, updated_at=Now()
-    )
+        changed |= changed_comics
+        changed |= bulk_folders_deleted(library, task.dirs_deleted)
+        changed |= bulk_comics_deleted(library, task.files_deleted)
+        cache.clear()
+    finally:
+        _finish_apply(library)
     # Wait to start the search index update in case more updates are incoming.
-    delayed_search_task = DelayedTasks(2, (SearchIndexUpdateTask(False),))
+    delayed_search_task = DelayedTasks(2, (SearchIndexJanitorUpdateTask(False),))
     LIBRARIAN_QUEUE.put(delayed_search_task)
 
-    if new_failed_imports:
-        text = "FAILED_IMPORTS"
-    else:
-        text = "LIBRARY_UPDATE_DONE"
-    LIBRARIAN_QUEUE.put(AdminNotifierTask(text))
     if changed:
-        LIBRARIAN_QUEUE.put(BroadcastNotifierTask("LIBRARY_CHANGED"))
+        LIBRARIAN_QUEUE.put(LIBRARY_CHANGED_TASK)
         elapsed_time = time.time() - start_time
-        LOG.info(f"Updated library {library.path} in {precisedelta(elapsed_time)}.")
+        LOG.info(
+            f"Updated library {library.path} in {precisedelta(int(elapsed_time))}."
+        )
         suffix = ""
         if imported_count:
             cps = int(imported_count / elapsed_time)
             suffix = f" at {cps} comics per second."
         LOG.verbose(f"Imported {imported_count} comics{suffix}.")
+    if new_failed_imports:
+        LIBRARIAN_QUEUE.put(FAILED_IMPORTS_TASK)
 
 
 class Updater(QueuedThread):
@@ -197,7 +233,7 @@ class Updater(QueuedThread):
 
     def process_item(self, task):
         """Run the updater."""
-        if isinstance(task, DBDiffTask):
+        if isinstance(task, UpdaterDBDiffTask):
             _apply(task)
         else:
             LOG.warning(f"Bad task sent to library updater {task}")
