@@ -2,29 +2,31 @@
 from multiprocessing import Process
 from time import sleep
 
+from codex.darwin_mp import force_darwin_multiprocessing_fork
 from codex.librarian.covers.coverd import CoverCreator
+from codex.librarian.covers.tasks import CoverTask
+from codex.librarian.db.tasks import UpdaterTask
 from codex.librarian.db.updaterd import Updater
 from codex.librarian.janitor.crond import Crond, janitor
-from codex.librarian.queue_mp import (
-    LIBRARIAN_QUEUE,
-    ComicCoverTask,
-    DelayedTasks,
-    JanitorTask,
-    NotifierTask,
-    PollLibrariesTask,
+from codex.librarian.janitor.tasks import JanitorTask
+from codex.librarian.queue_mp import LIBRARIAN_QUEUE, DelayedTasks
+from codex.librarian.search.searchd import SearchIndexer
+from codex.librarian.search.tasks import (
     SearchIndexerTask,
     SearchIndexRebuildIfDBChangedTask,
-    UpdaterTask,
-    WatchdogEventTask,
-    WatchdogSyncTask,
 )
-from codex.librarian.searchd import SearchIndexer
 from codex.librarian.watchdog.eventsd import EventBatcher
 from codex.librarian.watchdog.observers import (
     LibraryEventObserver,
     LibraryPollingObserver,
 )
-from codex.notifier import Notifier
+from codex.librarian.watchdog.tasks import (
+    WatchdogEventTask,
+    WatchdogPollLibrariesTask,
+    WatchdogSyncTask,
+)
+from codex.notifier.notifierd import Notifier
+from codex.notifier.tasks import NotifierTask
 from codex.settings.logging import get_logger
 from codex.threads import QueuedThread
 
@@ -54,24 +56,24 @@ class LibrarianDaemon(Process):
     proc = None
 
     def __init__(self):
-        """Create threads and thread pool."""
+        """Init process."""
         super().__init__(name=self.NAME, daemon=False)
 
     def _process_task(self, task):
         """Process an individual task popped off the queue."""
         run = True
-        if isinstance(task, ComicCoverTask):
+        if isinstance(task, CoverTask):
             self.cover_creator.queue.put(task)
         elif isinstance(task, WatchdogEventTask):
             self.event_batcher.queue.put(task)
         elif isinstance(task, UpdaterTask):
             self.updater.queue.put(task)
-        elif isinstance(task, NotifierTask):
+        elif isinstance(task, NotifierTask) and Notifier.thread:
             Notifier.thread.queue.put(task)
         elif isinstance(task, WatchdogSyncTask):
             for observer in self._observers:
                 observer.sync_library_watches()
-        elif isinstance(task, PollLibrariesTask):
+        elif isinstance(task, WatchdogPollLibrariesTask):
             self.library_polling_observer.poll(task.library_ids, task.force)
         elif isinstance(task, SearchIndexerTask):
             self.search_indexer.queue.put(task)
@@ -88,6 +90,7 @@ class LibrarianDaemon(Process):
 
     def _create_threads(self):
         """Create all the threads."""
+        force_darwin_multiprocessing_fork()
         self.delayed_tasks = DelayedTasksThread()
         self.cover_creator = CoverCreator()
         self.search_indexer = SearchIndexer()
@@ -113,17 +116,19 @@ class LibrarianDaemon(Process):
 
     def _start_threads(self):
         """Start all librarian's threads."""
-        LOG.debug("Starting Librarian threads.")
+        LOG.debug(f"Starting all {self.NAME} threads.")
         for thread in self._threads:
             thread.start()
-        LOG.debug("Started Librarian threads.")
+        LOG.debug(f"Started all {self.NAME} threads.")
 
     def _stop_threads(self):
         """Stop all librarian's threads."""
-        LOG.debug("Stopping Librarain threads...")
+        LOG.debug(f"Stopping all {self.NAME} threads...")
         for thread in reversed(self._threads):
             thread.stop()
-        LOG.debug("Stopped Librarian threads.")
+        for thread in reversed(self._threads):
+            thread.join()
+        LOG.debug(f"Stopped all {self.NAME} threads.")
 
     def run(self):
         """
@@ -163,7 +168,11 @@ class LibrarianDaemon(Process):
     @classmethod
     def shutdown(cls):
         """Stop the librarian process."""
+        if not cls.proc:
+            LOG.warning(f"Cannot shutdown {cls.NAME}. It hasn't started.")
+            return
         LIBRARIAN_QUEUE.put(cls.SHUTDOWN_TASK)
-        if cls.proc:
-            LOG.debug("Waiting to shut down librarian...")
-            cls.proc.join()
+        LOG.debug(f"Waiting for {cls.NAME} to join...")
+        cls.proc.join()
+        cls.proc = None
+        LOG.debug(f"{cls.NAME}joined.")
