@@ -3,9 +3,7 @@ import calendar
 import datetime
 import os
 
-from decimal import Decimal
 from pathlib import Path
-from uuid import uuid4
 
 from django.conf import settings
 from django.contrib.auth.models import Group
@@ -30,17 +28,23 @@ from django.db.models import (
     TextField,
     URLField,
 )
+from django.db.models.enums import TextChoices
 from django.utils.translation import gettext_lazy as _
 
+from codex.librarian.covers.status import CoverStatusTypes
+from codex.librarian.db.status import ImportStatusTypes
+from codex.librarian.janitor.status import JanitorStatusTypes
+from codex.librarian.search.status import SearchIndexStatusTypes
+from codex.librarian.watchdog.status import WatchdogStatusTypes
 from codex.serializers.choices import CHOICES
 from codex.settings.logging import get_logger
-from codex.settings.settings import XAPIAN_INDEX_PATH, XAPIAN_INDEX_UUID_PATH
 
 
 LOG = get_logger(__name__)
 
 
-SCHEMA_VERSION = 1
+MAX_PATH_LENGTH = 4095
+MAX_NAME_LENGTH = 128
 
 
 class BaseModel(Model):
@@ -48,7 +52,6 @@ class BaseModel(Model):
 
     created_at = DateTimeField(auto_now_add=True)
     updated_at = DateTimeField(auto_now=True)
-    # deleted_at = DateTimeField(null=True)
 
     class Meta:
         """Without this a real table is created and joined to."""
@@ -61,24 +64,14 @@ class BrowserGroupModel(BaseModel):
     """Browser groups."""
 
     DEFAULT_NAME = ""
+    ORDERING = ("name", "pk")
 
-    name = CharField(db_index=True, max_length=64, default=DEFAULT_NAME)
-    sort_name = CharField(db_index=True, max_length=130, default=DEFAULT_NAME)
-
-    def presave(self):
-        """Save the sort name. Called by save()."""
-        self.sort_name = self.name
-
-    def save(self, *args, **kwargs):
-        """Save the sort name as the name by default."""
-        self.presave()
-        super().save(*args, **kwargs)
+    name = CharField(db_index=True, max_length=MAX_NAME_LENGTH, default=DEFAULT_NAME)
 
     class Meta:
         """Without this a real table is created and joined to."""
 
         abstract = True
-        ordering = ["sort_name"]
 
 
 class Publisher(BrowserGroupModel):
@@ -93,6 +86,8 @@ class Publisher(BrowserGroupModel):
 class Imprint(BrowserGroupModel):
     """A Publishing imprint."""
 
+    ORDERING = ("publisher__name", "name", "pk")
+
     publisher = ForeignKey(Publisher, on_delete=CASCADE)
 
     class Meta:
@@ -100,10 +95,6 @@ class Imprint(BrowserGroupModel):
 
         unique_together = ("name", "publisher")
 
-    def presave(self):
-        """Save the sort name. Called by save()."""
-        self.sort_name = f"{self.publisher.name} {self.name}"
-
 
 class Series(BrowserGroupModel):
     """The series the comic belongs to."""
@@ -122,15 +113,13 @@ class Series(BrowserGroupModel):
 class Volume(BrowserGroupModel):
     """The volume of the series the comic belongs to."""
 
+    ORDERING = ("series__name", "name", "pk")
+
     publisher = ForeignKey(Publisher, on_delete=CASCADE)
     imprint = ForeignKey(Imprint, on_delete=CASCADE)
     series = ForeignKey(Series, on_delete=CASCADE)
     issue_count = PositiveSmallIntegerField(null=True)
 
-    def presave(self):
-        """Save the sort name. Called by save()."""
-        self.sort_name = f"{self.series.name} {self.name}"
-
     class Meta:
         """Constraints."""
 
@@ -150,14 +139,16 @@ class Library(BaseModel):
     DEFAULT_POLL_EVERY = datetime.timedelta(seconds=DEFAULT_POLL_EVERY_SECONDS)
 
     path = CharField(
-        unique=True, db_index=True, max_length=128, validators=[validate_dir_exists]
+        unique=True,
+        db_index=True,
+        max_length=MAX_PATH_LENGTH,
+        validators=[validate_dir_exists],
     )
     events = BooleanField(db_index=True, default=True)
     poll = BooleanField(db_index=True, default=True)
     poll_every = DurationField(default=DEFAULT_POLL_EVERY)
     last_poll = DateTimeField(null=True)
     update_in_progress = BooleanField(default=False)
-    schema_version = PositiveSmallIntegerField(default=0)
     groups = ManyToManyField(Group, blank=True)
 
     def __str__(self):
@@ -173,7 +164,7 @@ class Library(BaseModel):
 class NamedModel(BaseModel):
     """A for simple named tables."""
 
-    name = CharField(db_index=True, max_length=64)
+    name = CharField(db_index=True, max_length=MAX_NAME_LENGTH)
 
     class Meta:
         """Defaults to uniquely named, must be overridden."""
@@ -254,7 +245,7 @@ class WatchedPath(BrowserGroupModel):
     """A filesystem path with data for Watchdog."""
 
     library = ForeignKey(Library, on_delete=CASCADE, db_index=True)
-    path = CharField(max_length=4095, db_index=True)
+    path = CharField(max_length=MAX_PATH_LENGTH, db_index=True)
     stat = JSONField(null=True)
     parent_folder = ForeignKey(
         "Folder",
@@ -272,7 +263,6 @@ class WatchedPath(BrowserGroupModel):
         st[0] = st_record.st_mode
         st[1] = st_record.st_ino
         # st_dev changes every time with docker
-        # st[2] = st_record.st_dev
         st[6] = st_record.st_size
         st[8] = st_record.st_mtime
         self.stat = st
@@ -292,9 +282,24 @@ class Folder(WatchedPath):
     """File system folder."""
 
 
+def validate_file_format_choice(choice):
+    """Validate file format."""
+    values = Comic.FileFormat.values
+    if choice not in values:
+        raise ValidationError(_(f"{choice} is not one of {values}"))
+
+
 class Comic(WatchedPath):
     """Comic metadata."""
 
+    class FileFormat(TextChoices):
+        """Identifiers for file formats."""
+
+        COMIC = "comic"
+        PDF = "pdf"
+
+    ORDERING = ("series__name", "volume__name", "issue", "issue_suffix", "name", "pk")
+
     # From BaseModel, but Comics are sorted by these so index them
     created_at = DateTimeField(auto_now_add=True, db_index=True)
     updated_at = DateTimeField(auto_now=True, db_index=True)
@@ -305,9 +310,8 @@ class Comic(WatchedPath):
     )
 
     # Unique comic fields
-    issue = DecimalField(
-        db_index=True, decimal_places=2, max_digits=10, default=Decimal(0.0)
-    )
+    issue = DecimalField(db_index=True, decimal_places=2, max_digits=10, null=True)
+    issue_suffix = CharField(db_index=True, max_length=16, default="")
     volume = ForeignKey(Volume, db_index=True, on_delete=CASCADE)
     series = ForeignKey(Series, db_index=True, on_delete=CASCADE)
     imprint = ForeignKey(Imprint, db_index=True, on_delete=CASCADE)
@@ -332,11 +336,10 @@ class Comic(WatchedPath):
     country = CharField(db_index=True, max_length=32, null=True)
     language = CharField(db_index=True, max_length=32, null=True)
     # misc
-    cover_image = CharField(max_length=256, null=True)
-    format = CharField(db_index=True, max_length=16, null=True)
+    format = CharField(db_index=True, max_length=32, null=True)
     page_count = PositiveSmallIntegerField(db_index=True, default=0)
     read_ltr = BooleanField(db_index=True, default=True)
-    scan_info = CharField(max_length=128, null=True)
+    scan_info = CharField(max_length=MAX_NAME_LENGTH, null=True)
     web = URLField(null=True)
     # ManyToMany
     characters = ManyToManyField(Character)
@@ -348,27 +351,30 @@ class Comic(WatchedPath):
     tags = ManyToManyField(Tag)
     teams = ManyToManyField(Team)
     # Ignore these, they seem useless:
-    #
-    # black_and_white = BooleanField(default=False)
-    # price = DecimalField(decimal_places=2, max_digits=9, null=True)
-    # rights = CharField(max_length=64, null=True)
-    # manga = BooleanField(default=False)
-    # last_mark = PositiveSmallIntegerField(null=True)
+    # black_and_white
+    # last_mark
+    # manga
+    # price
+    # rights
     #
     # These are potentially useful, but too much work right now:
-    #
-    # is_version_of = CharField(max_length=64, null=True)
-    # alternate_issue = DecimalField(decimal_places=2, max_digits=6, null=True)
-    # alternate_volumes = ManyToManyField(Volume, related_name="alternate_volume")
-    # identifier = CharField(max_length=64, null=True)
+    # alternate_issue
+    # alternate_volumes
+    # cover_image
+    # identifier
+    # is_version_of
 
     # codex only
-    cover_path = CharField(max_length=4095)
     date = DateField(db_index=True, null=True)
     decade = PositiveSmallIntegerField(db_index=True, null=True)
     folders = ManyToManyField(Folder)
     max_page = PositiveSmallIntegerField(default=0)
     size = PositiveIntegerField(db_index=True)
+    file_format = CharField(
+        choices=FileFormat.choices,
+        max_length=max((len(val) for val in FileFormat.values)),
+        default=FileFormat.COMIC.value,
+    )
 
     class Meta:
         """Constraints."""
@@ -408,22 +414,12 @@ class Comic(WatchedPath):
         self._set_decade()
         self.max_page = max(self.page_count - 1, 0)
         self.size = Path(self.path).stat().st_size
-        sort_names = (self.volume.sort_name, f"{self.issue:06.1f}")
-        self.sort_name = " ".join(sort_names)
 
     def save(self, *args, **kwargs):
         """Save computed fields."""
         self.presave()
         super().save(*args, **kwargs)
 
-    def _get_display_issue(self):
-        """Get the issue number, even if its a half issue."""
-        if self.issue % 1 == 0:
-            issue_str = f"#{int(self.issue):0>3d}"
-        else:
-            issue_str = f"#{self.issue:05.1f}"
-        return issue_str
-
     def __str__(self):
         """Most common text representation for logging."""
         names = []
@@ -431,15 +427,19 @@ class Comic(WatchedPath):
             names.append(self.series.name)
         if self.volume.name:
             names.append(self.volume.name)
-        names.append(self._get_display_issue())
+        if self.issue is not None:
+            names.append(f"#{self.issue:06.1f}")
+        if self.issue_suffix:
+            names.append(self.issue_suffix)
         if self.name:
             names.append(self.name)
-        return " ".join(names)
+        return " ".join(names).strip()
 
 
 class AdminFlag(NamedModel):
     """Flags set by administrators."""
 
+    # TODO: SPA admin maybe change this to proper TextChoices.
     ENABLE_FOLDER_VIEW = "Enable Folder View"
     ENABLE_REGISTRATION = "Enable Registration"
     ENABLE_NON_USERS = "Enable Non Users"
@@ -475,7 +475,6 @@ def cascade_if_user_null(collector, field, sub_objs, _using):
             source=field.remote_field.model,
             source_attr=field.name,
             nullable=field.null,
-            # fail_on_restricted=False,
         )
 
     # Set them all to null
@@ -486,8 +485,10 @@ def cascade_if_user_null(collector, field, sub_objs, _using):
 
 def validate_fit_to_choice(choice):
     """Validate fit to choice."""
-    if choice is not None and choice not in CHOICES["fitTo"]:
-        raise ValidationError(_(f"{choice} is not one of $(FIT_TO_CHOICE_VALUES)"))
+    # Choices is loaded after migration time and after definition time.
+    values = CHOICES["fitTo"]
+    if choice is not None and choice not in values:
+        raise ValidationError(_(f"{choice} is not one of {values}"))
 
 
 class UserBookmark(BaseModel):
@@ -503,7 +504,10 @@ class UserBookmark(BaseModel):
     bookmark = PositiveSmallIntegerField(db_index=True, null=True)
     finished = BooleanField(default=False, db_index=True)
     fit_to = CharField(
-        validators=[validate_fit_to_choice], null=True, default=None, max_length=6
+        validators=[validate_fit_to_choice],
+        null=True,
+        default=None,
+        max_length=len("SCREEN"),
     )
     two_pages = BooleanField(default=None, null=True)
 
@@ -516,15 +520,13 @@ class UserBookmark(BaseModel):
 class FailedImport(WatchedPath):
     """Failed Comic Imports. Displayed in Admin Panel."""
 
-    MAX_REASON_LEN = 32
-
     def set_reason(self, exc):
         """Can't do this in save() because it breaks update_or_create."""
         reason = str(exc)
         suffixes = (f": {self.path}", f": '{self.path}'")
         for suffix in suffixes:
             reason = reason.removesuffix(suffix)
-        reason = reason[: self.MAX_REASON_LEN]
+        reason = reason[:MAX_NAME_LENGTH]
         self.name = reason.strip()
 
     class Meta:
@@ -533,55 +535,12 @@ class FailedImport(WatchedPath):
         unique_together = ("library", "path")
 
 
-class LatestVersion(BaseModel):
-    """Latest codex version."""
-
-    CODEX_VERSION_PK = 1
-    XAPIAN_INDEX_VERSION_PK = 2
-    version = CharField(max_length=32)
-
-    @classmethod
-    def _update_or_create(cls, pk, version):
-        search_kwargs = {"pk": pk}
-        defaults = {"version": version}
-        cls.objects.update_or_create(defaults=defaults, **search_kwargs)
-
-    @classmethod
-    def set_codex_version(cls, version):
-        """Ensure a single database row."""
-        cls._update_or_create(cls.CODEX_VERSION_PK, version)
-
-    @classmethod
-    def set_xapian_index_version(cls):
-        """Set the codex db to xapian matching id."""
-        version = str(uuid4())
-        cls._update_or_create(cls.XAPIAN_INDEX_VERSION_PK, version)
-        XAPIAN_INDEX_PATH.mkdir(parents=True, exist_ok=True)
-        with XAPIAN_INDEX_UUID_PATH.open("w") as uuid_file:
-            uuid_file.write(version)
-
-    @classmethod
-    def is_xapian_uuid_match(cls):
-        """Is this xapian index for this database."""
-        result = False
-        try:
-            with XAPIAN_INDEX_UUID_PATH.open("r") as uuid_file:
-                version = uuid_file.read()
-            lv = cls.objects.only("pk").get(
-                pk=cls.XAPIAN_INDEX_VERSION_PK, version=version
-            )
-            result = lv.pk == cls.XAPIAN_INDEX_VERSION_PK
-        except (FileNotFoundError, cls.DoesNotExist):
-            pass
-        except Exception as exc:
-            LOG.exception(exc)
-        return result
-
-
 class SearchQuery(Model):
     """Search queries."""
 
-    text = CharField(db_index=True, unique=True, max_length=256)
+    QUERY_MAX_LENGTH = 256
+
+    text = CharField(db_index=True, unique=True, max_length=QUERY_MAX_LENGTH)
     used_at = DateTimeField(auto_now_add=True, db_index=True)
 
 
@@ -606,3 +565,55 @@ class SearchResult(Model):
 
         unique_together = ("query", "comic")
         indexes = [Index(fields=["comic", "query"])]
+
+
+class LibrarianStatus(NamedModel):
+    """Active Library Tasks."""
+
+    DEFAULT_PARAMS = {
+        "name": "",
+        "preactive": False,
+        "complete": 0,
+        "total": 0,
+        "active": None,
+    }
+    TYPES = (
+        *CoverStatusTypes.values(),
+        *ImportStatusTypes.values(),
+        *JanitorStatusTypes.values(),
+        *SearchIndexStatusTypes.values(),
+        *WatchdogStatusTypes.values(),
+    )
+    type = CharField(db_index=True, max_length=32)
+    preactive = BooleanField(default=False)
+    complete = PositiveSmallIntegerField(default=0)
+    total = PositiveSmallIntegerField(default=0)
+    active = DateTimeField(null=True, default=None)
+
+    class Meta:
+        """Constraints."""
+
+        unique_together = ("type", "name")
+
+
+class Timestamp(NamedModel):
+    """Timestamp."""
+
+    COVERS = "covers"
+    JANITOR = "janitor"
+    SEARCH_INDEX = "search_index"
+    CODEX_VERSION = "codex_version"
+    XAPIAN_INDEX_UUID = "xapian_index_uuid"
+    NAMES = (COVERS, JANITOR, SEARCH_INDEX, CODEX_VERSION, XAPIAN_INDEX_UUID)
+
+    version = CharField(max_length=32, null=True, default=None)
+
+    @classmethod
+    def touch(cls, name):
+        """Touch a timestamp."""
+        cls.objects.get(name=name).save()
+
+    @classmethod
+    def get(cls, name):
+        """Get the timestamp."""
+        return cls.objects.get(name=name).updated_at.timestamp()
