@@ -4,10 +4,9 @@
 from pathlib import Path
 
 from filelock import FileLock
+from haystack.backends import BaseEngine
 from xapian_backend import XapianSearchBackend, XapianSearchQuery
 
-from codex._vendor.haystack.backends import BaseEngine
-
 
 def filelocked(func):
     """Decorate filelocked methods."""
