@@ -24,17 +24,19 @@ class UserBookmarkFinishedSerializer(ModelSerializer):
         fields = ("finished",)
 
 
+# TODO apiv3 move to session serializer and rename to ReaderSettings
 class ComicReaderSettingsSerializer(Serializer):
     """Reader settings the user can change."""
 
-    fitTo = ChoiceField(  # noqa: N815
+    fit_to = ChoiceField(
         choices=FIT_TO_CHOICES,
         allow_null=True,
         required=False,
     )
-    twoPages = BooleanField(allow_null=True, required=False)  # noqa: N815
+    two_pages = BooleanField(allow_null=True, required=False)
 
 
+# TODO apiv3 rework this terrible thing
 class ComicReaderBothSettingsSerializer(Serializer):
     """For both the default and local comic settings."""
 
