@@ -15,7 +15,7 @@ from rest_framework.serializers import (
 )
 
 from codex.serializers.choices import CHOICES, VUETIFY_NULL_CODE
-from codex.serializers.mixins import BrowserAggregateSerializerMixin
+from codex.serializers.mixins import UNIONFIX_PREFIX, BrowserCardOPDSBaseSerializer
 
 
 VUETIFY_NULL_CODE_STR = str(VUETIFY_NULL_CODE)
@@ -82,6 +82,7 @@ class FilterListField(ListField, ABC):
         super().__init__(
             *args,
             child=self.CHILD_CLASS(allow_null=True, **kwargs),
+            required=False,
             validators=self.VALIDATORS,
         )
 
@@ -116,22 +117,30 @@ class CharListField(FilterListField):
 class BrowserSettingsFilterSerializer(Serializer):
     """Filter values for settings."""
 
-    bookmark = ChoiceField(  # noqa: N815
-        choices=tuple(CHOICES["bookmarkFilter"].keys())
+    bookmark = ChoiceField(
+        choices=tuple(CHOICES["bookmarkFilter"].keys()), required=False
     )
     # Dynamic filters
-    community_rating = DecimalListField(max_digits=4, decimal_places=2)
+    community_rating = DecimalListField(
+        max_digits=5,
+        decimal_places=2,
+        coerce_to_string=False,
+    )
     characters = IntListField()
     country = CharListField()
     creators = IntListField()
-    critical_rating = DecimalListField(max_digits=4, decimal_places=2)
+    critical_rating = DecimalListField(
+        max_digits=5,
+        decimal_places=2,
+        coerce_to_string=False,
+    )
     decade = DecadeListField()
     format = CharListField()
     genres = IntListField()
     language = CharListField()
     locations = IntListField()
     age_rating = CharListField()
-    read_ltr = ListField(child=BooleanField())
+    read_ltr = ListField(child=BooleanField(), required=False)
     series_groups = IntListField()
     story_arcs = IntListField()
     tags = IntListField()
@@ -147,26 +156,34 @@ class BrowserSettingsSerializer(Serializer):
     It is also sent to the browser as part of BrowserOpenedSerializer.
     """
 
-    filters = BrowserSettingsFilterSerializer()
-    autoquery = CharField(allow_blank=True)
-    topGroup = ChoiceField(choices=tuple(CHOICES["topGroup"].keys()))  # noqa: N815
-    orderBy = ChoiceField(choices=tuple(CHOICES["orderBy"].keys()))  # noqa: N815
-    orderReverse = BooleanField()  # noqa: N815
-    show = BrowserSettingsShowGroupFlagsSerializer()
+    filters = BrowserSettingsFilterSerializer(required=False)
+    autoquery = CharField(allow_blank=True, required=False)
+    top_group = ChoiceField(choices=tuple(CHOICES["topGroup"].keys()), required=False)
+    order_by = ChoiceField(choices=tuple(CHOICES["orderBy"].keys()), required=False)
+    order_reverse = BooleanField(required=False)
+    show = BrowserSettingsShowGroupFlagsSerializer(required=False)
 
 
-class BrowserCardSerializer(BrowserAggregateSerializerMixin, Serializer):
+class BrowserCardSerializer(BrowserCardOPDSBaseSerializer):
     """Browse card displayed in the browser."""
 
-    pk = IntegerField(read_only=True)
-    group = CharField(read_only=True, max_length=1)
-    cover_path = CharField(read_only=True)
-    publisher_name = CharField(read_only=True)
-    series_name = CharField(read_only=True)
-    volume_name = CharField(read_only=True)
-    name = CharField(read_only=True)
-    issue = DecimalField(max_digits=5, decimal_places=1, read_only=True)
-    order_value = CharField(read_only=True)
+    publisher_name = CharField(
+        read_only=True, source=UNIONFIX_PREFIX + "publisher_name"
+    )
+    series_name = CharField(read_only=True, source=UNIONFIX_PREFIX + "series_name")
+    volume_name = CharField(read_only=True, source=UNIONFIX_PREFIX + "volume_name")
+    name = CharField(read_only=True, source=UNIONFIX_PREFIX + "name")
+    issue = DecimalField(
+        max_digits=16,
+        decimal_places=3,
+        read_only=True,
+        coerce_to_string=False,
+        source=UNIONFIX_PREFIX + "issue",
+    )
+    issue_suffix = CharField(read_only=True, source=UNIONFIX_PREFIX + "issue_suffix")
+    order_value = CharField(read_only=True, source=UNIONFIX_PREFIX + "order_value")
+    page_count = IntegerField(read_only=True, source=UNIONFIX_PREFIX + "page_count")
+    read_ltr = BooleanField(read_only=True, source=UNIONFIX_PREFIX + "read_ltr")
 
 
 class BrowserRouteSerializer(Serializer):
@@ -177,18 +194,18 @@ class BrowserRouteSerializer(Serializer):
     page = IntegerField(read_only=True)
 
 
-class BrowserFormChoicesSerializer(Serializer):
+class BrowserAdminFlagsSerializer(Serializer):
     """These choices change with browse context."""
 
-    enableFolderView = BooleanField(read_only=True)  # noqa: N815
+    enable_folder_view = BooleanField(read_only=True)
 
 
 class BrowserTitleSerializer(Serializer):
     """Elements for constructing the browse title."""
 
-    parentName = CharField(read_only=True, allow_null=True)  # noqa: N815
-    groupName = CharField(read_only=True)  # noqa: N815
-    groupCount = IntegerField(read_only=True, allow_null=True)  # noqa: N815
+    parent_name = CharField(read_only=True, allow_null=True)
+    group_name = CharField(read_only=True)
+    group_count = IntegerField(read_only=True, allow_null=True)
 
 
 class BrowserPageSerializer(Serializer):
@@ -196,20 +213,25 @@ class BrowserPageSerializer(Serializer):
 
     NUM_AUTOCOMPLETE_QUERIES = 10
 
-    browserTitle = BrowserTitleSerializer(read_only=True)  # noqa: N815
-    modelGroup = CharField(read_only=True)  # noqa: N815
-    upRoute = BrowserRouteSerializer(allow_null=True)  # noqa: N815
-    objList = ListField(  # noqa: N815
-        child=BrowserCardSerializer(read_only=True),
-        allow_empty=True,
-        read_only=True,
+    browser_title = BrowserTitleSerializer(read_only=True)
+    model_group = CharField(read_only=True)
+    up_route = BrowserRouteSerializer(allow_null=True, read_only=True)
+    obj_list = ListField(
+        child=BrowserCardSerializer(read_only=True), allow_empty=True, read_only=True
     )
-    numPages = IntegerField(read_only=True)  # noqa: N815
-    formChoices = BrowserFormChoicesSerializer(read_only=True)  # noqa: N815
-    librariesExist = BooleanField(read_only=True)  # noqa: N815
+    issue_max = DecimalField(
+        max_digits=16,
+        decimal_places=3,
+        read_only=True,
+        coerce_to_string=False,
+    )
+    num_pages = IntegerField(read_only=True)
+    admin_flags = BrowserAdminFlagsSerializer(read_only=True)
+    libraries_exist = BooleanField(read_only=True)
     queries = ListField(
         child=CharField(read_only=True), allow_empty=True, read_only=True
     )
+    covers_timestamp = IntegerField(read_only=True)
 
 
 class VersionsSerializer(Serializer):
@@ -223,5 +245,5 @@ class BrowserOpenedSerializer(Serializer):
     """Component open settings."""
 
     settings = BrowserSettingsSerializer(read_only=True)
-    browserPage = BrowserPageSerializer(read_only=True)  # noqa: N815
+    browser_page = BrowserPageSerializer(read_only=True)
     versions = VersionsSerializer(read_only=True)
