@@ -6,7 +6,6 @@ from multiprocessing import Queue
 from queue import Empty
 from threading import Thread
 
-from codex.darwin_mp import force_darwin_multiprocessing_fork
 from codex.settings.logging import get_logger
 
 
@@ -19,7 +18,17 @@ class BreakLoopError(Exception):
     pass
 
 
-class QueuedThread(Thread, ABC):
+class NamedThread(Thread, ABC):
+    """A thread that sets its name for ps."""
+
+    NAME = "abstract-named-thread"
+
+    def run_start(self):
+        """First thing to do when running a new thread."""
+        LOG.verbose(f"Started {self.NAME} thread")
+
+
+class QueuedThread(NamedThread, ABC):
     """Abstract Thread worker for doing queued tasks."""
 
     SHUTDOWN_MSG = "shutdown"
@@ -58,15 +67,14 @@ class QueuedThread(Thread, ABC):
 
     def run(self):
         """Run thread loop."""
-        force_darwin_multiprocessing_fork()
-        LOG.verbose(f"Started {self.NAME} thread")
+        self.run_start()
         while True:
             try:
                 self._check_item()
             except BreakLoopError:
                 break
             except Exception as exc:
-                LOG.error(f"Error in {self.NAME}:")
+                LOG.error(f"{self.NAME} crashed:")
                 LOG.exception(exc)
         LOG.verbose(f"Stopped {self.NAME} thread")
 
@@ -76,14 +84,16 @@ class QueuedThread(Thread, ABC):
 
     def join(self):
         """End the thread."""
+        LOG.debug(f"Waiting for {self.NAME} to join.")
         super().join(self.SHUTDOWN_TIMEOUT)
+        LOG.debug(f"{self.NAME} joined.")
 
 
 class AggregateMessageQueuedThread(QueuedThread, ABC):
     """Abstract Thread worker for buffering and aggregating messages."""
 
-    FLOOD_DELAY = 2
-    MAX_DELAY = 15
+    FLOOD_DELAY = 1
+    MAX_DELAY = 5
 
     def __init__(self, *args, **kwargs):
         """Initialize the cache."""
