@@ -17,37 +17,30 @@ from django.conf import settings
 from django.contrib import admin
 from django.contrib.staticfiles.storage import staticfiles_storage
 from django.urls import include, path, re_path
-from django.views.decorators.cache import cache_page
 from django.views.generic.base import RedirectView
 
-from codex.views.frontend import IndexView, browserconfig, webmanifest
+from codex.views.frontend import IndexView
 
 
-CACHE_TIME = 60 * 60
+TIMEOUT = 60 * 60
 
 urlpatterns = [
     path(
         "favicon.ico",
-        cache_page(CACHE_TIME)(
-            RedirectView.as_view(url=staticfiles_storage.url("img/favicon.ico"))
-        ),
+        RedirectView.as_view(url=staticfiles_storage.url("img/logo-32.webp")),
         name="favicon",
     ),
     path(
         "robots.txt",
-        cache_page(CACHE_TIME)(
-            RedirectView.as_view(url=staticfiles_storage.url("robots.txt"))
-        ),
+        RedirectView.as_view(url=staticfiles_storage.url("robots.txt")),
         name="robots",
     ),
-    path(
-        "browserconfig.xml", cache_page(CACHE_TIME)(browserconfig), name="browserconfig"
-    ),
-    path("site.webmanifest", cache_page(CACHE_TIME)(webmanifest), name="webmanifest"),
     path("api/v2/", include("codex.urls_api_v2")),
-    path("admin/", admin.site.urls, name="admin"),
+    path("opds/", include("codex.urls_opds")),
+    path("admin/", admin.site.urls),
+    path("", include("codex.urls_pwa")),
     path("", IndexView.as_view(), name="app"),
-    re_path(".*", IndexView.as_view(), name="app"),
+    re_path(".*", IndexView.as_view()),
 ]
 
 
