@@ -32,6 +32,8 @@ def set_timezone(request, serializer):
 class IsAuthenticatedOrEnabledNonUsers(IsAuthenticated):
     """Custom DRF Authentication class."""
 
+    code = 403
+
     def has_permission(self, request, view):
         """Return True if ENABLE_NON_USERS is true or user authenticated."""
         enu_flag = AdminFlag.objects.only("on").get(name=AdminFlag.ENABLE_NON_USERS)
@@ -69,7 +71,7 @@ class RegisterView(APIView):
         )
         LOG.verbose(f"Created user {username}")
         if count:
-            LOG.verbose("Linked user to existing session bookmarks")
+            LOG.verbose(f"Linked user to {count} existing session bookmarks")
         return user
 
     def validate(self):
@@ -110,7 +112,7 @@ class RegisterView(APIView):
         er_flag_on = (
             AdminFlag.objects.only("on").get(name=AdminFlag.ENABLE_REGISTRATION).on
         )
-        data = {"enableRegistration": er_flag_on}
+        data = {"enable_registration": er_flag_on}
         serializer = RegistrationEnabledSerializer(data)
         return Response(serializer.data)
 
@@ -128,6 +130,7 @@ class LoginView(APIView):
             raise ex
         return serializer
 
+    # TODO apiv3 put?
     def post(self, request, *args, **kwargs):
         """Authenticate and login."""
         serializer = self.validate()
@@ -154,6 +157,7 @@ class LoginView(APIView):
 class UserView(APIView):
     """User info."""
 
+    # TODO apiv3 GET
     def post(self, request, *args, **kwargs):
         """Get the user info for the current user."""
         serializer = TimezoneSerializer(data=self.request.data)
@@ -173,6 +177,7 @@ class LogoutView(APIView):
 
     permission_classes = [IsAuthenticated]
 
+    # TODO apiv3 put
     def post(self, request, *args, **kwargs):
         """Logout."""
         logout(request)
