@@ -1,12 +1,22 @@
 """Views for browsing comic library."""
+import json
+
 from copy import deepcopy
 from typing import Optional, Union
 
 from django.core.paginator import EmptyPage, Paginator
-from django.db.models import CharField, F, IntegerField, Value
+from django.db.models import F, Max, Value
+from django.db.models.fields import (
+    BooleanField,
+    CharField,
+    DateField,
+    DecimalField,
+    IntegerField,
+)
+from djangorestframework_camel_case.settings import api_settings
+from djangorestframework_camel_case.util import underscoreize
 from rest_framework.exceptions import ValidationError
 from rest_framework.response import Response
-from stringcase import camelcase
 
 from codex.exceptions import SeeOtherRedirectError
 from codex.models import (
@@ -18,6 +28,7 @@ from codex.models import (
     Publisher,
     SearchQuery,
     Series,
+    Timestamp,
     Volume,
 )
 from codex.serializers.browser import (
@@ -25,7 +36,7 @@ from codex.serializers.browser import (
     BrowserPageSerializer,
     BrowserSettingsSerializer,
 )
-from codex.serializers.mixins import UNIONFIX_PREFIX
+from codex.serializers.opds_v1 import BROWSER_CARD_AND_OPDS_ORDERED_UNIONFIX_VALUES_MAP
 from codex.settings.logging import get_logger
 from codex.version import PACKAGE_NAME, VERSION, get_latest_version
 from codex.views.auth import IsAuthenticatedOrEnabledNonUsers
@@ -46,34 +57,11 @@ class BrowserView(BrowserMetadataBaseView):
     _NAV_GROUPS = "rpisv"
     _MAX_OBJ_PER_PAGE = 100
     _ORPHANS = int(_MAX_OBJ_PER_PAGE / 20)
-    _BROWSER_CARD_FIELDS = [
-        "bookmark",
-        "child_count",
-        f"{UNIONFIX_PREFIX}cover_path",
-        "finished",
-        "group",
-        f"{UNIONFIX_PREFIX}issue",
-        "library",
-        "name",
-        "order_value",
-        "path",
-        "pk",
-        "progress",
-        "publisher_name",
-        "series_name",
-        "sort_name",
-        "volume_name",
-    ]
-    _BROWSER_SETTINGS_KEYS_SNAKE_CAMEL_MAP = {
-        snake_key: camelcase(snake_key)
-        for snake_key in BrowserMetadataBaseView.SESSION_DEFAULTS[
-            BrowserMetadataBaseView.BROWSER_KEY
-        ].keys()
-    }
-    _BROWSER_SETTINGS_KEYS_CAMEL_SNAKE_MAP = {
-        v: k for k, v in _BROWSER_SETTINGS_KEYS_SNAKE_CAMEL_MAP.items()
-    }
-    _NONE_CHARFIELD = Value(None, CharField())
+    _NONE_DECIMALFIELD = Value(None, DecimalField())
+    _EMPTY_CHARFIELD = Value("", CharField())
+    _ZERO_INTEGERFIELD = Value(0, IntegerField())
+    _NONE_BOOLFIELD = Value(None, BooleanField())
+    _NONE_DATEFIELD = Value(None, DateField())
 
     _GROUP_INSTANCE_SELECT_RELATED = {
         Comic: ("series", "volume"),
@@ -81,11 +69,12 @@ class BrowserView(BrowserMetadataBaseView):
         Series: (None,),
         Imprint: ("publisher",),
         Publisher: (None,),
+        Folder: ("parent_folder",),
     }
     _DEFAULT_ROUTE = {
         "name": "browser",
         "params": BrowserMetadataBaseView.SESSION_DEFAULTS[
-            BrowserMetadataBaseView.BROWSER_KEY
+            BrowserMetadataBaseView.SESSION_KEY
         ]["route"],
     }
 
@@ -118,22 +107,22 @@ class BrowserView(BrowserMetadataBaseView):
         #######################
         if model not in (Folder, Comic):
             # For folder view stability sorting compatibility
-            queryset = queryset.annotate(library=Value(0, IntegerField()))
+            queryset = queryset.annotate(library=self._ZERO_INTEGERFIELD)
 
         #######################
         # Sortable aggregates #
         #######################
-        order_by = self.params.get("order_by", self.DEFAULT_ORDER_KEY)
-        order_func = self.get_aggregate_func(order_by, is_model_comic, autoquery_pk)
+        order_key = self.get_order_key()
+        order_func = self.get_aggregate_func(order_key, model, autoquery_pk)
         queryset = queryset.annotate(order_value=order_func)
 
         ########################
         # Annotate name fields #
         ########################
         # Optimized to only lookup what is used on the frontend
-        publisher_name = self._NONE_CHARFIELD
-        series_name = self._NONE_CHARFIELD
-        volume_name = self._NONE_CHARFIELD
+        publisher_name = self.NONE_CHARFIELD
+        series_name = self.NONE_CHARFIELD
+        volume_name = self.NONE_CHARFIELD
 
         if is_model_comic:
             series_name = F("series__name")
@@ -148,17 +137,22 @@ class BrowserView(BrowserMetadataBaseView):
             series_name=series_name,
             volume_name=volume_name,
         )
-        if is_model_comic:
-            issue = F("issue")
-        else:
-            issue = Value(None, IntegerField())
-        queryset = queryset.annotate(
-            **{f"{UNIONFIX_PREFIX}issue": issue},
-        )
-        if model not in (Folder, Comic):
-            path = self._NONE_CHARFIELD
+        if not is_model_comic:
             queryset = queryset.annotate(
-                path=path,
+                issue=self._NONE_DECIMALFIELD,
+                issue_suffix=self._EMPTY_CHARFIELD,
+            )
+        if model not in (Folder, Comic):
+            queryset = queryset.annotate(path=self.NONE_CHARFIELD)
+
+        if not is_model_comic:
+            queryset = queryset.annotate(
+                read_ltr=self._NONE_BOOLFIELD,
+                size=self._ZERO_INTEGERFIELD,
+                date=self._NONE_DATEFIELD,
+                summary=self._EMPTY_CHARFIELD,
+                comments=self._EMPTY_CHARFIELD,
+                notes=self._EMPTY_CHARFIELD,
             )
 
         return queryset
@@ -168,15 +162,15 @@ class BrowserView(BrowserMetadataBaseView):
         # the model group shown must be:
         #   A valid nav group or 'c'
         #   the child of the current nav group or 'c'
-        if self.kwargs["group"] == "f":
-            return "f"
-        if (
-            self.valid_nav_group_index == len(self.valid_nav_groups) - 1
-            or self.valid_nav_group_index is None
-        ):
+        group = self.kwargs["group"]
+        if group == self.FOLDER_GROUP:
+            return self.FOLDER_GROUP
+        if group == self.valid_nav_groups[-1]:
             # special case for lowest valid group
-            return "c"
-        next_valid_nav_group = self.valid_nav_groups[self.valid_nav_group_index + 1]
+            return self.COMIC_GROUP
+        next_valid_nav_group = self.valid_nav_groups[
+            self.valid_nav_groups.index(group) + 1
+        ]
         return next_valid_nav_group
 
     def _set_browse_model(self):
@@ -199,9 +193,14 @@ class BrowserView(BrowserMetadataBaseView):
         folder_list = self._add_annotations(folder_list, Folder, autoquery_pk)
         comic_list = self._add_annotations(comic_list, Comic, autoquery_pk)
 
-        # Reduce to values to make union align columns correctly
-        folder_list = folder_list.values(*self._BROWSER_CARD_FIELDS)
-        comic_list = comic_list.values(*self._BROWSER_CARD_FIELDS)
+        # Create ordered annotated values to make union align columns correctly because
+        # django lacks a way to specify values column order.
+        folder_list = folder_list.values(
+            **BROWSER_CARD_AND_OPDS_ORDERED_UNIONFIX_VALUES_MAP
+        )
+        comic_list = comic_list.values(
+            **BROWSER_CARD_AND_OPDS_ORDERED_UNIONFIX_VALUES_MAP
+        )
 
         obj_list = folder_list.union(comic_list)
         return obj_list
@@ -218,8 +217,8 @@ class BrowserView(BrowserMetadataBaseView):
         obj_list = self._add_annotations(obj_list, self.model, autoquery_pk)
 
         # Convert to a dict because otherwise the folder/comic union blows
-        # up the paginator
-        obj_list = obj_list.values(*self._BROWSER_CARD_FIELDS)
+        # up the paginator. Use the same annotations for the serializer.
+        obj_list = obj_list.values(**BROWSER_CARD_AND_OPDS_ORDERED_UNIONFIX_VALUES_MAP)
 
         return obj_list
 
@@ -230,9 +229,9 @@ class BrowserView(BrowserMetadataBaseView):
 
         # Recall root id & relative path from way back in
         # object creation
-        if self.host_folder:
-            if self.host_folder.parent_folder:
-                up_pk = self.host_folder.parent_folder.pk
+        if self.group_instance and isinstance(self.group_instance, Folder):
+            if self.group_instance.parent_folder:
+                up_pk = self.group_instance.parent_folder.pk
             else:
                 up_pk = 0
 
@@ -241,36 +240,37 @@ class BrowserView(BrowserMetadataBaseView):
     def _set_group_instance(self):
         """Create group_class instance."""
         pk = self.kwargs.get("pk")
-        group = self.kwargs.get("group")
-        self.group_instance: Optional[Union[Folder, Publisher, Imprint, Series, Volume]]
-        if pk == 0:
-            self.group_instance = None
-        elif group == self.FOLDER_GROUP:
-            self.group_instance = self.host_folder
-        else:
-            if not self.group_class:
-                raise ValueError("No group_class set in browser")
-            try:
-                select_related = self._GROUP_INSTANCE_SELECT_RELATED[self.group_class]
-                self.group_instance = self.group_class.objects.select_related(
-                    *select_related
-                ).get(pk=pk)
-            except self.group_class.DoesNotExist:
-                self._raise_redirect({"group": group}, f"{group}={pk} does not exist!")
+        self.group_instance: Optional[
+            Union[Folder, Publisher, Imprint, Series, Volume]
+        ] = None
+        if not pk:
+            return
+        try:
+            select_related = self._GROUP_INSTANCE_SELECT_RELATED[self.group_class]
+            self.group_instance = self.group_class.objects.select_related(
+                *select_related
+            ).get(pk=pk)
+        except self.group_class.DoesNotExist:
+            group = self.kwargs.get("group")
+            self._raise_redirect({"group": group}, f"{group}={pk} does not exist!")
 
     def _get_browse_up_route(self):
         """Get the up route from the first valid ancestor."""
         # Ancestor group
-        ancestor_group = None
-        if self.valid_nav_group_index is not None and self.valid_nav_group_index > 0:
-            ancestor_group = self.valid_nav_groups[self.valid_nav_group_index - 1]
+        group = self.kwargs.get("group")
+        if self.valid_nav_groups.index(group) > 0:
+            ancestor_group = self.valid_nav_groups[
+                self.valid_nav_groups.index(group) - 1
+            ]
+        else:
+            ancestor_group = None
 
         up_group = ancestor_group
         up_pk = None
 
         # Ancestor pk
         pk = self.kwargs.get("pk")
-        if up_group == "r" or pk == 0:
+        if up_group == self.ROOT_GROUP or pk == 0:
             up_pk = 0
         elif up_group:
             # get the ancestor pk from the current group
@@ -307,8 +307,7 @@ class BrowserView(BrowserMetadataBaseView):
                     prefix = self.group_instance.library.path
                     if prefix[-1] != "/":
                         prefix += "/"
-                    user = self.request.user
-                    if not user or not getattr(user, "is_staff"):  # noqa: B009
+                    if not self.is_admin():
                         # remove library path for not admins
                         parent_name = parent_name.removeprefix(prefix)
                     suffix = "/" + self.group_instance.name
@@ -317,186 +316,25 @@ class BrowserView(BrowserMetadataBaseView):
             group_name = self.group_instance.name
 
         browser_page_title = {
-            "parentName": parent_name,
-            "groupName": group_name,
-            "groupCount": group_count,
+            "parent_name": parent_name,
+            "group_name": group_name,
+            "group_count": group_count,
         }
 
         return browser_page_title
 
-    def _get_valid_top_groups(self):
-        """
-        Get valid top groups for the current settings.
-
-        Valid top groups are determined by the Browser Settings.
-        """
-        valid_top_groups = []
-
-        for nav_group in self._NAV_GROUPS:
-            if self.params["show"].get(nav_group):
-                valid_top_groups.append(nav_group)
-        # Issues is always a valid top group
-        valid_top_groups += ["c"]
-
-        return valid_top_groups
-
-    def _set_valid_nav_groups(self, valid_top_groups):
-        """
-        Get valid nav gorups for the current settings.
-
-        Valid nav groups are the top group and below that are also
-        enabled in browser settings.
-
-        May always navigate to root 'r' nav group.
-        """
-        top_group = self.params["top_group"]
-        nav_group = self.kwargs["group"]
-        self.valid_nav_group_index = None
-        possible_nav_groups = valid_top_groups
-
-        for possible_index, possible_nav_group in enumerate(possible_nav_groups):
-            if top_group in (possible_nav_group, "c"):
-                if top_group == "c":
-                    tail_top_groups = []
-                else:
-                    # all the nav groups past this point, but not 'c' the last one
-                    tail_top_groups = possible_nav_groups[possible_index:-1]
-                self.valid_nav_groups = ["r"] + tail_top_groups
-                for valid_index, valid_nav_group in enumerate(self.valid_nav_groups):
-                    if nav_group == valid_nav_group:
-                        self.valid_nav_group_index = valid_index
-                break
-
-    def _raise_redirect(self, route_changes, reason):
-        """Redirect the client to a valid group url."""
-        route = deepcopy(self._DEFAULT_ROUTE)
-        route["params"].update(route_changes)
-        settings = self._get_serializer_settings()
-        detail = {"route": route, "settings": settings, "reason": reason}
-        raise SeeOtherRedirectError(detail=detail)
-
-    def _validate_folder_settings(self):
-        """Check that all the view variables for folder mode are set right."""
-        # Check folder view admin flag
-        try:
-            flag = AdminFlag.objects.only("on").get(name=AdminFlag.ENABLE_FOLDER_VIEW)
-            enable_folder_view = flag.on
-        except AdminFlag.DoesNotExist:
-            enable_folder_view = False
-        if not enable_folder_view:
-            new_top_group = "r"
-            self.params["top_group"] = new_top_group
-            self.save_params_to_session()
-            reason = "folder view disabled"
-            route_changes = {"group": new_top_group}
-            self._raise_redirect(route_changes, reason)
-
-        # Validate folder nav group
-        # Redirect if nav group is wrong
-        group = self.kwargs["group"]
-        if group != "f":
-            route_changes = {"group": "f"}
-            reason = f"{group=} does not match top_group 'f'"
-            self._raise_redirect(route_changes, reason)
-        self.valid_nav_groups = ["f"]
-        self.valid_nav_group_index = 0
-
-    def _validate_browser_group_settings(self):
-        """Check that all the view variables for browser mode are set right."""
-        top_group = self.params.get("top_group")
-        nav_group = self.kwargs["group"]
-
-        # Validate Browser top group
-        # Change top_group if its not in the valid top groups
-        valid_top_groups = self._get_valid_top_groups()
-        if top_group not in valid_top_groups:
-            reason = f"top group not in valid nav groups, changed {top_group}"
-            if nav_group in valid_top_groups:
-                valid_top_group = nav_group
-                reason += f"to nav group: {nav_group}"
-            else:
-                valid_top_group = valid_top_groups[0]
-                reason += f"to first valid top group {valid_top_groups[0]}"
-            self.params["top_group"] = valid_top_group
-            LOG.verbose(reason)
-            self.top_group_changed = True
-
-        # Validate Browser Nav Group
-        # Redirect if nav group is wrong
-        self._set_valid_nav_groups(valid_top_groups)
-        if self.valid_nav_group_index is None:
-            self.save_params_to_session()
-            new_nav_group = self.valid_nav_groups[0]
-            route_changes = {"group": new_nav_group}
-            reason = f"Nav group {nav_group} unavailable, redirect to {new_nav_group}"
-            self._raise_redirect(route_changes, reason)
-        pk = self.kwargs["pk"]
-        if nav_group == "r" and pk:
-            # r never has a pk
-            reason = f"Redirect r with {pk=} to pk 0"
-            self._raise_redirect({}, reason)
-        lowest_group = self.valid_nav_groups[-1]
-        if (
-            self.autoquery_first or (self.top_group_changed and nav_group == "r")
-        ) and nav_group != lowest_group:
-            # if not at the lowest issues showing nav group and its the first aq
-            # or the top group changed away from root.
-            route_changes = {"group": lowest_group}
-            if self.autoquery_first:
-                # params change handled in _apply_put, maybe doesn't need redirect.
-                reason = "first autoquery: show issues view"
-            else:
-                reason = f"changed top group so group {lowest_group} shows issues now"
-            self.save_params_to_session()
-            self._raise_redirect(route_changes, reason)
-
-    def _validate_settings(self):
-        """Validate group and top group settings."""
-        group = self.kwargs.get("group")
-        top_group = self.params.get("top_group")
-        if top_group == self.FOLDER_GROUP:
-            self._validate_folder_settings()
-        else:
-            self._validate_browser_group_settings()
-        # save route once validated.
-        pk = self.kwargs.get("pk")
-        page = self.kwargs.get("page")
-        # to save last browser route
-        self.params["route"] = {"group": group, "pk": pk, "page": page}
-
-    def _apply_put_settings(self, data):
-        """Validate submitted settings and apply them over the session settings."""
-        serializer = BrowserSettingsSerializer(data=data)
-        try:
-            serializer.is_valid(raise_exception=True)
-        except ValidationError as exc:
-            LOG.error(serializer.errors)
-            LOG.exception(exc)
-            raise exc
-
-        for key, value in serializer.validated_data.items():
-            snake_key = self._BROWSER_SETTINGS_KEYS_CAMEL_SNAKE_MAP[key]
-            if snake_key == "autoquery" and not self.params.get(snake_key) and value:
-                self.autoquery_first = True
-            if snake_key == "top_group" and self.params.get(snake_key) != value:
-                self.top_group_changed = True
-            self.params[snake_key] = value
-        if self.autoquery_first:
-            self.params["order_by"] = "search_score"
-            self.params["order_reverse"] = True
-
     def _page_out_out_bounds(self, page, num_pages):
         """Redirect page out of bounds."""
         group = self.kwargs.get("group")
         pk = self.kwargs.get("pk", 1)
-        route = {"group": group, "pk": pk}
+        if page > num_pages:
+            new_page = num_pages
+        else:
+            new_page = 1
+        route_changes = {"group": group, "pk": pk, "page": new_page}
         reason = f"{page=} does not exist!"
-        if page < 1:
-            route["page"] = 1
-        elif page > num_pages:
-            route["page"] = num_pages
-        LOG.verbose(f"{reason} redirect to {route}.")
-        self._raise_redirect(route, reason)
+        LOG.verbose(f"{reason} redirect to page {new_page}.")
+        self._raise_redirect(route_changes, reason)
 
     def _paginate(self, queryset):
         """Paginate the queryset into a final object list."""
@@ -513,17 +351,17 @@ class BrowserView(BrowserMetadataBaseView):
 
     def _get_browser_page(self):
         """Validate settings and get the querysets."""
-        self._validate_settings()
         self._set_browse_model()
+        self._set_group_instance()  # Placed up here to invalidate earlier
         # Create the main query with the filters
+        is_model_comic = self.model == Comic
         try:
-            object_filter, autoquery_pk = self.get_query_filters(
-                self.model == Comic, False
-            )
+            object_filter, autoquery_pk = self.get_query_filters(is_model_comic, False)
         except Folder.DoesNotExist:
             pk = self.kwargs.get("pk")
             self._raise_redirect(
-                {"group": "f"}, f"folder {pk} Does not exist! Redirect to root folder."
+                {"group": self.FOLDER_GROUP},
+                f"folder {pk} Does not exist! Redirect to root folder.",
             )
         group = self.kwargs.get("group")
 
@@ -533,17 +371,12 @@ class BrowserView(BrowserMetadataBaseView):
             queryset = self._get_browser_group_queryset(object_filter, autoquery_pk)
 
         # Order
-        order_by = self.get_order_by(self.model)
-        queryset = queryset.order_by(*order_by)
+        queryset = self.get_order_by(self.model, queryset)
 
         # Paginate
         obj_list, num_pages = self._paginate(queryset)
 
-        # Save the session
-        self.save_params_to_session()
-
         # get additional context
-        self._set_group_instance()
         if group == self.FOLDER_GROUP:
             up_group, up_pk = self._get_folder_up_route()
         else:
@@ -563,55 +396,232 @@ class BrowserView(BrowserMetadataBaseView):
             "text", flat=True
         )[: BrowserPageSerializer.NUM_AUTOCOMPLETE_QUERIES]
 
+        if is_model_comic:
+            # runs obj list query twice :/
+            issue_max = obj_list.aggregate(Max("issue"))["issue__max"]
+        else:
+            issue_max = 0
+
+        covers_timestamp = Timestamp.get(Timestamp.COVERS)
+
         # construct final data structure
         browser_page = {
-            "upRoute": up_route,
-            "browserTitle": browser_page_title,
-            "modelGroup": self.model_group,
-            "objList": obj_list,
-            "numPages": num_pages,
-            "formChoices": {"enableFolderView": efv_flag.on},
-            "librariesExist": libraries_exist,
+            "up_route": up_route,
+            "browser_title": browser_page_title,
+            "model_group": self.model_group,
+            "obj_list": obj_list,
+            "issue_max": issue_max,
+            "num_pages": num_pages,
+            "admin_flags": {"enable_folder_view": efv_flag.on},
+            "libraries_exist": libraries_exist,
             "queries": queries,
+            "covers_timestamp": covers_timestamp,
         }
         return browser_page
 
-    def _load_params(self, from_request=False):
-        """Load self.params from the session, update with request."""
-        self.top_group_changed = False
-        self.autoquery_first = False
-        self.load_params_from_session()
-        if from_request:
-            self._apply_put_settings(self.request.data)
+    def _get_valid_top_groups(self):
+        """
+        Get valid top groups for the current settings.
 
-    def put(self, request, *args, **kwargs):
+        Valid top groups are determined by the Browser Settings.
+        """
+        valid_top_groups = []
+
+        for nav_group in self._NAV_GROUPS:
+            if self.params["show"].get(nav_group):
+                valid_top_groups.append(nav_group)
+        # Issues is always a valid top group
+        valid_top_groups += [self.COMIC_GROUP]
+
+        return valid_top_groups
+
+    def _set_valid_browse_nav_groups(self, valid_top_groups):
+        """
+        Get valid nav groups for the current settings.
+
+        Valid nav groups are the top group and below that are also
+        enabled in browser settings.
+
+        May always navigate to root 'r' nav group.
+        """
+        top_group = self.params["top_group"]
+        nav_group = self.kwargs["group"]
+        self.valid_nav_groups = (self.ROOT_GROUP,)
+
+        for possible_index, possible_nav_group in enumerate(valid_top_groups):
+            if top_group in (possible_nav_group, self.COMIC_GROUP):
+                # all the nav groups past this point, but not 'c' the last one
+                tail_top_groups = valid_top_groups[possible_index:-1]
+
+                self.valid_nav_groups = (*self.valid_nav_groups, *tail_top_groups)
+                if nav_group not in self.valid_nav_groups:
+                    route_changes = {"group": self.ROOT_GROUP}
+                    reason = (
+                        f"Nav group {nav_group} unavailable, "
+                        f"redirect to {self.ROOT_GROUP}"
+                    )
+                    self._raise_redirect(route_changes, reason)
+                break
+
+    def _raise_redirect(self, route_mask, reason):
+        """Redirect the client to a valid group url."""
+        route = deepcopy(self._DEFAULT_ROUTE)
+        # TODO apiv3 params and DEFAULT ROUTE is too GUI ORIENTED.
+        # CHANGE. FLATTEN INTO SETTINGS?
+        route["params"].update(route_mask)
+        detail = {"route": route, "settings": self.params, "reason": reason}
+        raise SeeOtherRedirectError(detail=detail)
+
+    def _validate_folder_settings(self, enable_folder_view):
+        """Check that all the view variables for folder mode are set right."""
+        # Check folder view admin flag
+        if not enable_folder_view:
+            self.params["top_group"] = self.ROOT_GROUP
+            reason = "folder view disabled"
+            self._raise_redirect({}, reason)
+
+        top_group = self.params["top_group"]
+        if top_group != self.FOLDER_GROUP:
+            self.params["top_group"] = self.FOLDER_GROUP
+            reason = f"top_group {top_group} doesn't match route {self.FOLDER_GROUP}"
+            self._raise_redirect({"group": self.FOLDER_GROUP}, reason)
+
+        # set valid folder nav groups
+        self.valid_nav_groups = (self.FOLDER_GROUP,)
+
+    def _validate_browser_group_settings(self):
+        """Check that all the view variables for browser mode are set right."""
+        nav_group = self.kwargs["group"]
+        top_group = self.params.get("top_group")
+        pk = self.kwargs["pk"]
+
+        # Validate Browser top_group
+        # Change top_group if its not in the valid top groups
+        valid_top_groups = self._get_valid_top_groups()
+        if top_group not in valid_top_groups:
+            reason = f"top_group {top_group} not in valid nav groups, changed to "
+            if nav_group in valid_top_groups:
+                valid_top_group = nav_group
+                reason += "nav group: "
+            else:
+                valid_top_group = valid_top_groups[0]
+                reason += "first valid top group "
+            reason += valid_top_group
+            LOG.verbose(reason)
+            self.params["top_group"] = valid_top_group
+            page = self.kwargs["page"]
+            route = {"group": nav_group, "pk": pk, "page": page}
+            self._raise_redirect(route, reason)
+
+        # Validate Browser nav_group
+        # Redirect if nav group is wrong
+        self._set_valid_browse_nav_groups(valid_top_groups)
+
+        # Validate pk
+        if nav_group == self.ROOT_GROUP and pk:
+            # r never has a pk
+            reason = f"Redirect r with {pk=} to pk 0"
+            self._raise_redirect({"pk": 0}, reason)
+
+    def _validate_settings(self):
+        """Validate group and top group settings."""
+        group = self.kwargs.get("group")
+        order_key = self.get_order_key()
+        enable_folder_view = False
+        if group == self.FOLDER_GROUP or order_key == "path":
+            try:
+                enable_folder_view = (
+                    AdminFlag.objects.only("on")
+                    .get(name=AdminFlag.ENABLE_FOLDER_VIEW)
+                    .on
+                )
+            except Exception:
+                pass
+
+        if group == self.FOLDER_GROUP:
+            self._validate_folder_settings(enable_folder_view)
+        else:
+            self._validate_browser_group_settings()
+
+        # Validate path sort
+        if order_key == "path" and not enable_folder_view:
+            pk = self.kwargs("pk")
+            page = self.kwargs("page")
+            route_changes = {"group": group, "pk": pk, "page": page}
+            self.params["order_by"] = "sort_name"
+            reason = "order by path not allowed by admin flag."
+            self._raise_redirect(route_changes, reason)
+
+    def _parse_query_params(self, query_params):
+        """Parse GET query parameters: filter object & snake case."""
+        # TODO apiv3 read up on sending objects via GET
+        #      possibly use filter_bookmark syntax or something.
+        result = {}
+        for key, val in query_params.items():
+            if key == "filters":
+                parsed_val = json.loads(val)
+                if not parsed_val:
+                    continue
+            else:
+                parsed_val = val
+
+            result[key] = parsed_val
+        # TODO apiv3 do this for all GETS
+        result = underscoreize(result, **api_settings.JSON_UNDERSCOREIZE)
+
+        return result
+
+    def _parse_params(self):
+        """Validate submitted settings and apply them over the session settings."""
+        self.params = deepcopy(self.SESSION_DEFAULTS[self.SESSION_KEY])
+        if self.request.method == "GET":
+            data = self._parse_query_params(self.request.GET)
+        elif self.request.method == "PUT":
+            data = self._parse_query_params(self.request.data)
+        else:
+            data = None
+
+        if data:
+            serializer = BrowserSettingsSerializer(data=data)
+            try:
+                serializer.is_valid(raise_exception=True)
+            except ValidationError as exc:
+                LOG.error(serializer.errors)
+                LOG.exception(exc)
+                raise exc
+
+            # Overlay params on defaults.
+            for key, value in serializer.validated_data.items():
+                self.params[key] = value
+
+        self._validate_settings()
+
+    ##############
+    # DEPRECATED #
+    ##############
+    def put(self, _request, *args, **kwargs):
         """Create the view."""
-        self._load_params(from_request=True)
+        # TODO apiv3 Deprecated, remove
+        LOG.verbose("Browser put method is deprecated.")
+        self._parse_params()
         browser_page = self._get_browser_page()
 
         serializer = BrowserPageSerializer(browser_page)
+        self.save_params_to_session()
         return Response(serializer.data)
 
-    def _get_serializer_settings(self):
-        """Get the camelcase settings."""
-        settings = {}
-        for key in self._BROWSER_SETTINGS_KEYS_SNAKE_CAMEL_MAP:
-            camel_key = camelcase(key)
-            settings[camel_key] = self.params.get(key)
-        return settings
-
-    def get(self, request, *args, **kwargs):
+    def get(self, _request, *args, **kwargs):
         """Get browser settings."""
-        self._load_params()
+        # TODO apiv3 moves settings & version to another api.
+        self._parse_params()
         browser_page = self._get_browser_page()
-
-        settings = self._get_serializer_settings()
         latest_version = get_latest_version(PACKAGE_NAME)
 
         data = {
-            "settings": settings,
-            "browserPage": browser_page,
+            "settings": self.params,
+            "browser_page": browser_page,
             "versions": {"installed": VERSION, "latest": latest_version},
         }
         serializer = BrowserOpenedSerializer(data)
+        self.save_params_to_session()
         return Response(serializer.data)
