@@ -2,26 +2,28 @@
 from distutils.util import strtobool
 
 from dateutil.parser import parse as du_parse
+from django.contrib.auth.models import User
 from django.db.models import F, Q
 from django.db.models.functions import Now
+from haystack.query import SearchQuerySet
 from humanfriendly import parse_size
 from xapian_backend import DATETIME_FORMAT
 
-from codex._vendor.haystack.query import SearchQuerySet
-from codex.librarian.queue_mp import LIBRARIAN_QUEUE, SearchIndexUpdateTask
-from codex.models import Comic, Folder, SearchQuery, SearchResult
+from codex.librarian.queue_mp import LIBRARIAN_QUEUE
+from codex.librarian.search.tasks import SearchIndexJanitorUpdateTask
+from codex.models import Comic, SearchQuery, SearchResult
 from codex.settings.logging import get_logger
 from codex.views.group_filter import GroupACLMixin
-from codex.views.session import SessionView
+from codex.views.session import SessionViewBase
 
 
 LOG = get_logger(__name__)
 DATE_FORMAT = DATETIME_FORMAT.removesuffix("%H%M%S")
 RANGE_DELIMITER = ".."
-XAPIAN_UPPERCASE_OPERATORS = set(["AND", "OR", "XOR", "NEAR", "ADJ"])
+XAPIAN_UPPERCASE_OPERATORS = frozenset(["AND", "OR", "XOR", "NEAR", "ADJ"])
 
 
-class BrowserBaseView(SessionView, GroupACLMixin):
+class BrowserBaseView(SessionViewBase, GroupACLMixin):
     """Browse comics with a variety of filters and sorts."""
 
     _SEARCH_FIELD_ALIASES = {
@@ -42,9 +44,9 @@ class BrowserBaseView(SessionView, GroupACLMixin):
         "team": "teams",
         "updated": "updated_at",
     }
-    SESSION_KEY = SessionView.BROWSER_KEY
-    COMIC_GROUP = "c"
+    ROOT_GROUP = "r"
     FOLDER_GROUP = "f"
+    COMIC_GROUP = "c"
     GROUP_RELATION = {
         "p": "publisher",
         "i": "imprint",
@@ -53,10 +55,14 @@ class BrowserBaseView(SessionView, GroupACLMixin):
         COMIC_GROUP: "pk",
         FOLDER_GROUP: "parent_folder",
     }
+    SESSION_KEY = SessionViewBase.BROWSER_KEY
+    _BOOKMARK_FILTERS = ("UNREAD", "IN_PROGRESS", "READ")
 
     def __init__(self, *args, **kwargs):
         """Set params for the type checker."""
         super().__init__(*args, **kwargs)
+        self._is_admin = None
+        self.params = {}
 
     def _filter_by_comic_field(self, field, is_model_comic):
         """Filter by a comic any2many attribute."""
@@ -96,24 +102,41 @@ class BrowserBaseView(SessionView, GroupACLMixin):
             ubm_rel = "comic__" + ubm_rel
         return ubm_rel
 
+    def _get_userbookmark_filter(self, ubm_rel):
+        """Get a filter for my session or user defined bookmarks."""
+        if self.request.user.is_authenticated:
+            my_bookmarks_kwargs = {f"{ubm_rel}__user": self.request.user}
+        else:
+            my_bookmarks_kwargs = {
+                f"{ubm_rel}__session__session_key": self.request.session.session_key
+            }
+        return Q(**my_bookmarks_kwargs)
+
     def _get_bookmark_filter(self, is_model_comic, choice):
         """Build bookmark query."""
         if choice is None:
             choice = self.params["filters"].get("bookmark", "ALL")
-        bookmark_filter = Q()
-        if choice in (
-            "UNREAD",
-            "IN_PROGRESS",
-        ):
+        if choice in self._BOOKMARK_FILTERS:
             ubm_rel = self.get_ubm_rel(is_model_comic)
+            my_userbookmark_filter = self._get_userbookmark_filter(ubm_rel)
 
-            bookmark_filter &= (
-                Q(**{f"{ubm_rel}__finished": False})
-                | Q(**{ubm_rel: None})
-                | Q(**{f"{ubm_rel}__finished": None})
-            )
-            if choice == "IN_PROGRESS":
-                bookmark_filter &= Q(**{f"{ubm_rel}__bookmark__gt": 0})
+            if choice == "UNREAD":
+                bookmark_filter = Q(**{ubm_rel: None}) | Q(
+                    my_userbookmark_filter
+                    & Q(**{f"{ubm_rel}__finished__in": (False, None)})
+                )
+            elif choice == "IN_PROGRESS":
+                bookmark_filter = my_userbookmark_filter & Q(
+                    **{f"{ubm_rel}__bookmark__gt": 0}
+                )
+            elif choice == "READ":
+                bookmark_filter = my_userbookmark_filter & Q(
+                    **{f"{ubm_rel}__finished": True}
+                )
+            else:
+                bookmark_filter = Q()
+        else:
+            bookmark_filter = Q()
         return bookmark_filter
 
     def _get_folders_filter(self):
@@ -126,24 +149,16 @@ class BrowserBaseView(SessionView, GroupACLMixin):
 
         return folders_filter
 
-    def _get_parent_folder_filter(self):
-        """Create the folder and comic object lists."""
-        pk = self.kwargs.get("pk")
-        if pk:
-            self.host_folder = Folder.objects.select_related("parent_folder").get(pk=pk)
-        else:
-            self.host_folder = None
-
-        return Q(parent_folder=self.host_folder)
-
     def _get_browser_group_filter(self):
         """Get the objects we'll be displaying."""
         # Get the instances that are children of the group_instance
         # And the filtered comics that are children of the group_instance
         group_filter = Q()
         pk = self.kwargs.get("pk")
-        if pk:
-            group = self.kwargs.get("group")
+        group = self.kwargs.get("group")
+        if pk or group == self.FOLDER_GROUP:
+            if not pk:
+                pk = None
             group_relation = self.GROUP_RELATION[group]
             group_filter |= Q(**{group_relation: pk})
 
@@ -297,9 +312,8 @@ class BrowserBaseView(SessionView, GroupACLMixin):
                 )
                 search_results.append(sr)
             if search_engine_out_of_date:
-                # XXX This should not happen. Need to sync search engine better.
                 LOG.warning("Search index out of date. Scoring non-existent comics.")
-                task = SearchIndexUpdateTask(False)
+                task = SearchIndexJanitorUpdateTask(False)
                 LIBRARIAN_QUEUE.put(task)
             SearchResult.objects.bulk_create(search_results)
         except Exception as exc:
@@ -373,25 +387,28 @@ class BrowserBaseView(SessionView, GroupACLMixin):
             self.params["autoquery"] = " ".join(autoquery_tokens)
         except Exception as exc:
             LOG.warning(exc)
-            LOG.exception(exc)
         return search_filter, autoquery_pk
 
     def _get_group_filter(self, choices):
         """Get filter for the displayed group."""
         is_folder_view = self.kwargs.get("group") == self.FOLDER_GROUP
-        if is_folder_view:
-            if choices:
-                # Choice view needs to get all descendant comic attributes
-                # So filter by all the folders
-                group_filter = self._get_folders_filter()
-            else:
-                # Regular browsing just needs to filter by the parent
-                group_filter = self._get_parent_folder_filter()
+        if is_folder_view and choices:
+            # Choice view needs to get all descendant comic attributes
+            # So filter by all the folders
+            group_filter = self._get_folders_filter()
         else:
             # The browser filter is the same for all views
             group_filter = self._get_browser_group_filter()
+
         return group_filter
 
+    def is_admin(self):
+        """Is the current user an admin."""
+        if self._is_admin is None:
+            user = self.request.user
+            self._is_admin = user and isinstance(user, User) and user.is_staff
+        return self._is_admin
+
     def get_aggregate_filter(self, is_model_comic):
         """Return the filter for making aggregates."""
         bookmark_filter_join = self._get_bookmark_filter(is_model_comic, None)
@@ -399,16 +416,22 @@ class BrowserBaseView(SessionView, GroupACLMixin):
         aggregate_filter = bookmark_filter_join & comic_field_filter
         return aggregate_filter
 
-    def get_query_filters(self, is_model_comic, choices=False):
-        """Return the main object filter and the one for aggregates."""
-        object_filter = Q()
-
-        object_filter &= self.get_group_acl_filter(is_model_comic)
-
-        object_filter &= self._get_group_filter(choices)
+    def get_query_filters_without_group(self, is_model_comic):
+        """Return all the filters except the group filter."""
+        object_filter = self.get_group_acl_filter(is_model_comic)
 
         search_filter, autoquery_pk = self._get_search_filter(is_model_comic)
         object_filter &= search_filter
 
         object_filter &= self.get_aggregate_filter(is_model_comic)
         return object_filter, autoquery_pk
+
+    def get_query_filters(self, is_model_comic, choices=False):
+        """Return the main object filter and the one for aggregates."""
+        object_filter, autoquery_pk = self.get_query_filters_without_group(
+            is_model_comic
+        )
+
+        object_filter &= self._get_group_filter(choices)
+
+        return object_filter, autoquery_pk
